@@ -336,7 +336,7 @@ class DockerV:
         }
 
     @retry(errors.ClientAPIError, tries=3, timeout_secs=0.5, logger=logger)
-    def exec_cmd(self, cmd, user=None):
+    def exec_cmd(self, cmd, user=None, container_name=False):
         """Wraps and logs the `exec_create
         <https://docker-py.readthedocs.org/en/latest/api/#exec_create>`_ and `exec_start
         <https://docker-py.readthedocs.org/en/latest/api/#exec_start>`_ methods.
@@ -345,17 +345,18 @@ class DockerV:
         :returns: The command output.
         """
         logger.debug('exec_cmd: %s', cmd)
+        container_name = container_name or self.docker_id
         try:
-            exec_id = self.__cli.exec_create(self.docker_id, cmd, user=user)
+            exec_id = self.__cli.exec_create(container_name, cmd, user=user)
         except (docker.errors.APIError, docker.errors.NullResource) as error:
             error_msg = get_error_message(error)
             if 'is not running' in error_msg:
-                raise errors.NotRunning(self.docker_id)
+                raise errors.NotRunning(container_name)
             if 'Resource ID was not provided' in error_msg:
                 raise errors.CommandError(cmd, "Container does not exist")
             if hasattr(error, 'is_client_error') and error.is_client_error():
                 raise errors.ClientAPIError(
-                    "Container {0} is in conflict or stopped unexpectedly".format(self.docker_id))
+                    "Container {0} is in conflict or stopped unexpectedly".format(container_name))
             raise
         res = decode(self.__cli.exec_start(exec_id.get('Id')))
         return res
@@ -754,15 +755,8 @@ class DockerV:
             self.__docker_id = self.config.get("container_name").lower()
             self.remove_container()
         hconfig = self.__cli.create_host_config(**host_config)
-        if "5432/tcp" in hconfig.get('PortBindings'):
-            default_network_info = self.__cli.inspect_network('bridge')
-            default_gateway = default_network_info.get('IPAM').get('Config')[0].get('Gateway')
-            host_port = hconfig.get('PortBindings').get("5432/tcp")[0].get('HostPort')
-            hconfig.get('PortBindings').update({
-                "5432/tcp": [
-                    {'HostIp': '127.0.0.1', 'HostPort': host_port},
-                    {'HostIp': default_gateway, 'HostPort': host_port}
-                ]})
+        safe_ports = self._secure_port_bindings(hconfig.get('PortBindings'))
+        hconfig.get('PortBindings').update(safe_ports)
         logger.debug('Container config %s', container_config)
         try:
             container_obj = self.__cli.create_container(
@@ -789,6 +783,51 @@ class DockerV:
                     ' process, manual intervention is required. Error message:\n{msg}'
                     .format(msg=error.explanation))
 
+    def _secure_port_bindings(self, port_bindings):
+        """Checks the ports that will be exposed in a container and changes their bindings
+        so they are only exposed to localhost, Docker's default gateway, and any network
+        defined in the `listen_addresses` parameter in the `deployer` section of the
+        DeployV configuration file. This is to avoid security issues by exposing ports
+        to the public interfaces.
+
+        param port_bindings: Dict with the ports that will be exposed in the format:
+
+            {
+                "8069/tcp": [
+                    {'HostIp': '0.0.0.0', 'HostPort': '17006'},
+                ]
+            }
+
+        This dictionary is obtained from the PortBindings key in the hostconfig dictionary.
+
+        :return: Dictionary in the same format as the one received, but were all the ports
+        are exposed only to internal networks.
+
+            {
+                "8069/tcp": [
+                    {'HostIp': '172.17.0.1', 'HostPort': '17006'},
+                    {'HostIp': '127.0.0.1', 'HostPort': '17006'},
+                ]
+            }
+        """
+        res = {}
+        default_network_info = self.__cli.inspect_network('bridge')
+        default_gateway = default_network_info.get('IPAM').get('Config')[0].get('Gateway')
+        deploy_cfg = self._InstanceV__full_config.deployer
+        for port, bindings in port_bindings.items():
+            if port == "22/tcp":
+                continue
+            host_port = bindings[0].get('HostPort')
+            safe_bindings = [
+                {'HostIp': '127.0.0.1', 'HostPort': host_port},
+                {'HostIp': default_gateway, 'HostPort': host_port}
+            ]
+            if deploy_cfg.get('listen_addresses'):
+                for network in deploy_cfg.get('listen_addresses'):
+                    safe_bindings.append({'HostIp': network, 'HostPort': host_port})
+            res.update({port: safe_bindings})
+        return res
+
     def _get_used_ports(self):
         """Method that checks the ports of all the containers in the server
         in order to list all the ports used by instances in order to decide
