@@ -11,12 +11,27 @@ from functools import wraps
 from inspect import signature
 import abc
 import glob
-import imp
 import logging
 import os
 import six
 import extend_me
 
+try:
+    from imp import load_source
+except ModuleNotFoundError:
+    import importlib.util
+    import importlib.machinery
+
+    def load_source(modname, filename):
+        loader = importlib.machinery.SourceFileLoader(modname, filename)
+        spec = importlib.util.spec_from_file_location(modname, filename, loader=loader)
+        module = importlib.util.module_from_spec(spec)
+        # The module is always executed and not cached in sys.modules.
+        # Uncomment the following line to cache the module.
+        # sys.modules[module.__name__] = module
+        loader.exec_module(module)
+        return module
+
 
 logger = logging.getLogger(__name__)  # pylint: disable=C0103
 _eventlistener_meta = extend_me.ExtensibleByHashType._('EventListener', hashattr='name')
@@ -32,7 +47,7 @@ def load_extensions():
     path = os.path.dirname(os.path.realpath(__file__))
     for path in glob.glob(os.path.join(path, '..', 'extensions/[!_]*.py')):
         name, ext = os.path.splitext(os.path.basename(path))
-        modules[name] = imp.load_source(name, path)
+        modules[name] = load_source(name, path)
     return modules
 
 
