@@ -146,6 +146,15 @@ class PostgresShell:
         for key, value in self.__defaults.items():
             if not self.__config.get(key, False):
                 self.__config.update({key: value})
+        shell = spur.LocalShell()
+        for executable in ['pg_dump', 'pg_restore', 'psql']:
+            custom_exec = "/opt/orchestsh/bin/%s" % executable
+            # Negative expression used to receive a return code of 1 (True) if it exists.
+            exec_exists = shell.run(
+                shlex.split("test ! -f %s" % custom_exec),
+                allow_error=True
+            ).return_code
+            self.__config.update({executable: custom_exec if exec_exists else executable})
 
     def __format_cmd(self, cmd, items):
         res = cmd
@@ -206,10 +215,10 @@ class PostgresShell:
         if self.__config.get('password', False):
             os.environ['PGPASSWORD'] = self.__config.get('password')
         self.__config.update({'dbname': database})
-        dump_cmd = 'pg_dump {dbname} -w -O -x -f {0}'
+        dump_cmd = '{pg_dump} -d {dbname} -w -O -x -f {0}'
         if jobs:
             self.__config.update({'num': jobs})
-            dump_cmd = 'pg_dump -w  -j {num} -F d {dbname} -f {0} -x'
+            dump_cmd = '{pg_dump} -w  -j {num} -F d -d {dbname} -f {0} -x'
         if exclude:
             for table in exclude.split(","):
                 dump_cmd += " --exclude-table=%s " % (table)
@@ -303,9 +312,9 @@ class PostgresShell:
         if self.__config.get('password', False):
             os.environ['PGPASSWORD'] = self.__config.get('password')
         self.__config.update({'dbname': database_name})
-        restore_cmd = 'psql {dbname} -w -f {0}'
+        restore_cmd = '{psql} -d {dbname} -w -f {0}'
         if jobs:
-            restore_cmd = 'pg_restore -w -j {num} -F d {0} -d {dbname} -x -O -n public'
+            restore_cmd = '{pg_restore} -w -j {num} -F d {0} -d {dbname} -x -O -n public'
             cnf.update({'num': str(jobs)})
         restore_cmd = self.__format_cmd(restore_cmd, cnf)
         restore_cmd = restore_cmd.format(dump_name, **cnf)
