@@ -2,11 +2,14 @@
 import os
 import os.path as osp
 import logging
-import re
-import spur
 import sys
+from deployv.helpers.utils import parse_repo_url
+from branchesv.branches import load as load_branches
 
-_logger = logging.getLogger()
+_logger = logging.getLogger('CloneOCADeps')
+
+TRAVIS_REPO_SLUG = os.environ.get("TRAVIS_REPO_SLUG")
+TRAVIS_REPO_OWNER = TRAVIS_REPO_SLUG and TRAVIS_REPO_SLUG.split("/")[0]
 
 
 def parse_depfile(depfile, version, owner='vauxoo'):
@@ -21,20 +24,24 @@ def parse_depfile(depfile, version, owner='vauxoo'):
         if not line or line.startswith('#'):
             continue
         parts = line.split()
+        parts_count = len(parts)
         repo = parts[0]
-        if len(parts) > 2:
-            branch = parts[2]
-        else:
+        url = parts_count > 1 and parts[1]
+        branch = parts_count > 2 and parts[2]
+        commit = parts_count > 3 and parts[3]
+        if not branch:
             branch = version
-        if len(parts) > 1:
-            url = parts[1]
-        else:
+        if not url:
             url = 'https://github.com/%s/%s.git' % (owner, repo)
-        deps.append((repo, url, branch))
+        if commit == '#':
+            # It's not a commit, it's an inline comment
+            commit = False
+        deps.append((repo, url, branch, commit))
     return deps
 
 
-def git_checkout(deps_checkout_dir, reponame, url, branch):
+def git_checkout(deps_checkout_dir, reponame, url, branch, commit=False,
+                 tmp_folder=False, key_path=False):
     """ This method downloads the repositories found in the oca_dependencies
     files and clones them in the specified dir. If he specified directory
     where the dependencies will be cloned already exists
@@ -44,27 +51,22 @@ def git_checkout(deps_checkout_dir, reponame, url, branch):
     :param reponame: name for repository
     :param url: url address repository
     :param branch: branch or version repository
+    :param commit: Optional commit to clone
     :return: Returns list (full path)
     """
+    repo = {
+        'branch': branch, 'commit': commit, 'depth': 1, 'name': reponame,
+        'path': reponame, 'repo_url': {'origin': url}
+    }
+    res = load_branches([repo], deps_checkout_dir, tmp_folder, key_path)
+    if res:
+        return (False, res.get('msg'))
     checkout_dir = osp.join(deps_checkout_dir, reponame)
-    if not osp.isdir(checkout_dir):
-        command = ['git', 'clone', '-q', url, '-b', branch,
-                   '--depth=1', checkout_dir]
-        _logger.info('Calling %s', ' '.join(command))
-        shell = spur.LocalShell()
-        try:
-            shell.run(command)
-        except spur.RunProcessError as error:
-            message_str = ("Could not clone repo {repo} "
-                           "branch {branch}, error: {error} ").format(repo=url,
-                                                                      branch=branch,
-                                                                      error=error.stderr_output)
-            return (False, message_str)
-
     return (True, checkout_dir)
 
 
-def get_dep_filename(deps_checkout_dir, build_dir, file_name, version):
+def get_dep_filename(deps_checkout_dir, build_dir, file_name,
+                     version, tmp_folder, key_path=False):
     """ This method, makes a recursive search directories to get all the
     files listed in the parameters file_name (oca_dependencies.txt)
     :param deps_checkout_dir: he directory in which the dependency
@@ -79,23 +81,24 @@ def get_dep_filename(deps_checkout_dir, build_dir, file_name, version):
     depfilename = osp.join(build_dir, file_name)
     dependencies.append((depfilename, 'vauxoo'))
     for repo in os.listdir(deps_checkout_dir):
-        _logger.info('examining %s', repo)
+        _logger.debug('examining %s', repo)
         processed.add(repo)
         depfilename = osp.join(deps_checkout_dir, repo, file_name)
         dependencies.append((depfilename, 'vauxoo'))
     for dependency in dependencies:
         try:
+            owner = TRAVIS_REPO_OWNER or dependency[1]
             with open(dependency[0]) as depfile:
-                deps = parse_depfile(depfile, version, owner=dependency[1])
+                deps = parse_depfile(depfile, version, owner=owner)
         except IOError:
             deps = []
-        for depname, url, branch in deps:
-            _logger.info('* processing %s', depname)
+        for depname, url, branch, commit in deps:
+            _logger.debug('* processing %s', depname)
             if depname in processed:
                 continue
             processed.add(depname)
-            checkout_dir = git_checkout(deps_checkout_dir, depname,
-                                        url, branch)
+            checkout_dir = git_checkout(
+                deps_checkout_dir, depname, url, branch, commit, tmp_folder, key_path)
             if not checkout_dir[0]:
                 return checkout_dir
             new_dep = (osp.join(checkout_dir[1], file_name),
@@ -119,14 +122,11 @@ def get_owner_from_url(url):
     :type url: str
     :return: The organization of the specified repo (default: vauxoo)
     """
-    regex = r'(?:(git@|http[s]?:\/\/).*)[:\/](?P<org>.*)/(?:.*)'
-    match = re.match(regex, url)
-    if not match:
-        return 'vauxoo'
-    return match.groupdict().get('org')
+    url_parts = parse_repo_url(url)
+    return url_parts.get('namespace') or 'vauxoo'
 
 
-def run(deps_checkout_dir, build_dir, version):
+def run(deps_checkout_dir, build_dir, version, temp_folder, key_path=False):
     """ This method executes above methods
     :param deps_checkout_dir: he directory in which the dependency
     repositories will be cloned
@@ -134,9 +134,9 @@ def run(deps_checkout_dir, build_dir, version):
     repositories have been cloned
     """
     depens = get_dep_filename(deps_checkout_dir, build_dir, 'oca_dependencies.txt',
-                              version)
+                              version, temp_folder, key_path)
     return depens
 
 
 if __name__ == '__main__':
-    run(sys.argv[1], sys.argv[2], sys.argv[3])
+    run(*sys.argv[1:6])
