@@ -58,10 +58,15 @@ def generate_binds(volumes, working_folder):
     """
     volume_binds = {}
     for key, value in six.iteritems(volumes):
-        volume = path.join(path.expanduser(working_folder), key)
+        paths = value.split(':')
+        host_path = ''
+        instance_path = ''
+        if len(paths) > 1:
+            host_path, instance_path = paths[:]
+        volume = path.expanduser(host_path or path.join(working_folder, key))
         volume_binds.update({
             volume: {
-                "bind": value,
+                "bind": instance_path or value,
                 "ro": False,
             }
         })
