@@ -2,7 +2,7 @@ import tarfile
 import zipfile
 
 
-class DecompressHelper(object):
+class DecompressHelper:
     valid_extensions = ('tar.gz', 'tar.bz2', 'zip', 'tar')
 
     def __init__(self, filename):
