@@ -794,10 +794,16 @@ def get_ip_address(ifname):
     )[20:24])
 
 
-def is_docker_ip(ip_addr):
-    """Recevives an ip and checks if it is in the docker network range"""
+def is_docker_ip(ip_addr, docker_ip=False):
+    """Recevives an ip and checks if it is in the docker network range.
+    The optional `docker_ip` parameter is used to receive the docker gateway IP
+    from another source instead of obtaining it from the docker0 network. This is
+    useful in cases where the docker0 network doesn't exist, e.g: when we use deployv
+    inside a container.
+    """
     ip_addr = socket.gethostbyname(ip_addr)
-    docker0 = re.sub(r"\.\d{1,3}$", ".0/24", get_ip_address('docker0'))
+    docker_ip = docker_ip or get_ip_address('docker0')
+    docker0 = re.sub(r"\.\d{1,3}$", ".0/24", docker_ip)
     return ipaddress.ip_address(decode(ip_addr)) in ipaddress.ip_network(decode(docker0))
 
 
