@@ -15,3 +15,21 @@ SAAS_VERSIONS = {
     'saas-15': 'saas-15',
     'saas-17': 'saas-17',
 }
+
+PSQL_VERSIONS = {
+  '13.0': '11',
+  '12.0': '11',
+  '11.0': '9.6',
+  '10.0': '9.6',
+  '9.0': '9.6',
+  '8.0': '9.5'
+}
+
+TRAVIS_VERSIONS = {
+  '13.0': '3.6',
+  '12.0': '3.6',
+  '11.0': '3.5',
+  '10.0': '2.7',
+  '9.0': '2.7',
+  '8.0': '2.7'
+}
