@@ -143,6 +143,8 @@ class InstanceV(DockerV):
         # ports
         if self.config.get('full_stack', False):
             self.config.get('ports').update({'22': None})
+        if (self.config.get('full_stack', False) or
+                self.__full_config.backup_type in ['btrfs', 'zfs']):
             self.update_info_postgres_container()
         self.pull()
         logger.info('Starting Odoo container %s',
@@ -477,15 +479,26 @@ class InstanceV(DockerV):
                     db_name, image_name, modules)
         working_folder = path.join(self.config.get("working_folder"),
                                    self.config.get('container_name'))
+        data_dir = self.get_data_dir()
+        working_folder = data_dir.replace('filestore', '') if data_dir else working_folder
         db_config = self.db_config
-        entrypoint_exists = self.exec_cmd_image(
-            'test -x /entry_point', self.config.get('volumes'), working_folder, image=image_name)
-        entrypoint = '/entry_point' if not entrypoint_exists.get('error') else '/entry_point.py'
+        for entrypoint in ['/entry_point', '/entrypoint', '/entry_point.py']:
+            entrypoint_exists = self.exec_cmd_image(
+                'test -x %s' % entrypoint, self.config.get('volumes'),
+                working_folder, image=image_name)
+            if not entrypoint_exists.get('error'):
+                break
+        odoo_conf_exists = self.exec_cmd_image(
+            'test -f /home/odoo/.odoorc', self.config.get('volumes'),
+            working_folder, image=image_name)
+        odoo_conf_file = ('/home/odoo/.odoorc' if not odoo_conf_exists.get('error') else
+                          '/home/odoo/.openerp_serverrc')
         update_command = (
             "{entrypoint} run --user odoo '/home/odoo/instance/odoo/{odoo}"
-            " -c /home/odoo/.openerp_serverrc"
+            " -c {odoo_conf}"
             " -d {database} -u {{module}} --logfile {{log}} --stop-after-init'"
-            .format(entrypoint=entrypoint, odoo=self.odoo_binary, database=db_name))
+            .format(entrypoint=entrypoint, odoo=self.odoo_binary, database=db_name,
+                    odoo_conf=odoo_conf_file))
         pg_shell = PostgresShell(utils.odoo2postgres(db_config))
         for module in modules:
             for attempt in range(2):
@@ -540,6 +553,8 @@ class InstanceV(DockerV):
         image_name = self.config.get('image_name') or self.basic_info.get('image_name')
         working_folder = path.join(self.config.get("working_folder"),
                                    self.config.get('container_name'))
+        data_dir = self.get_data_dir()
+        working_folder = data_dir.replace('filestore', '') if data_dir else working_folder
         logger.info('Updating database %s using COU', db_name)
         entrypoint_exists = self.exec_cmd_image(
             'test -x /entry_point', self.config.get('volumes'), working_folder, image=image_name)
@@ -684,18 +699,21 @@ class InstanceV(DockerV):
             return {}
         ins_config = self.config.copy()
         psql_version = self.get_postgres_version()
-        psql_image = "postgres:{version}".format(version=psql_version)
+        psql_image = self.__full_config.psql_image or \
+            "postgres:{version}".format(version=psql_version)
         role = 'createdb'
+        self.config.get('volumes').update({'postgresql/data': '/var/lib/postgresql/data/pgdata'})
         if self.config.get('full_stack'):
             role = 'superuser'
             self.config.get('volumes').update({'pg_log': destination})
         self.config.update({"image_name": psql_image})
         self.pull()
-        self.config.update({'ports': {'5432': None}})
+        self.config.update({'ports': {'5432': self.db_config.get('db_export_port') or None}})
         self.config.get('env_vars').update({
             'db_host': '127.0.0.1',
             'db_port': 5432,
             'postgres_password': 'postgres',
+            'pgdata': '/var/lib/postgresql/data/pgdata',
             'pg_log_path': destination
         })
         self.config.update({"container_name": self.config.get('postgres_container_name')})
@@ -781,9 +799,20 @@ class InstanceV(DockerV):
         use_template = self.__full_config.container_config.get('use_template')
         jobs = self.__full_config.jobs or 5
         helper_obj = database_helper.DatabaseHelper.get_helper(use_template)
+        extensions = self.__full_config.postgres.get('extensions')
+        if (self.__full_config.instance_config.get('customer_id')
+                in self.__full_config.postgres.get('avoid_postgis', [])):
+            try:
+                extensions.remove('postgis')
+            except ValueError:
+                pass
+            try:
+                extensions.remove('postgis_topology')
+            except ValueError:
+                pass
         db_helper = helper_obj(
             utils.odoo2postgres(self.db_config), self._get_admin_config(),
-            self.__full_config.postgres.get('extensions'))
+            extensions)
         backup_dir = backup_dir or self.__full_config.backup_src
         res_search = db_helper.search_candidate(backup_dir, customer_id)
         if not res_search[0]:
@@ -803,11 +832,13 @@ class InstanceV(DockerV):
             self.__full_config, path.basename(res_search[1]), self.__full_config.prefix)
         if self.docker_id:
             self.stop_instance()
+        logger.info("Start restore database")
         create_res = db_helper.create_database(candidate, db_name, self.db_config.get('db_owner'),
                                                self.db_config.get('db_owner_passwd'), jobs=jobs)
         if not create_res[0]:
             res.update({'error': create_res[1]})
             return res
+        logger.info("Database restored")
         create_date = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
         if self.docker_id and path.exists(filestore):
             filestore_res = self.restore_filestore(filestore, db_name)
@@ -979,9 +1010,11 @@ class InstanceV(DockerV):
         :param src_folder: Full path to the folder that contains the filestore you want to restore
         :param database_name: Database name being restored
         """
+        logger.info("Restoring filestore")
         working_folder = path.join(self.config.get("working_folder"),
                                    self.config.get('container_name'))
         dest = path.join(path.expanduser(working_folder), 'filestore')
+        dest = self.get_data_dir() or dest
         volumes = {
             path.abspath(src_folder): {
                 'bind': '/tmp/filestore',
@@ -992,14 +1025,24 @@ class InstanceV(DockerV):
                 'ro': False
             }
         }
+        cmd_filestore = ("(ls {fsname} 2>/dev/null && cp -R /tmp/filestore/* {fsname}) "
+                         "|| cp -R /tmp/filestore {fsname}")
+        if self.__full_config.deployer.get('use_rsync'):
+            cmd_filestore = ("apt update && apt-get install rsync && "
+                             "rsync -rza /tmp/filestore/ {fsname}/")
         bash_lines = [
-            "rm -rf {fsname}",
+            # Add "/*" to the end of path because it trigger an error when it's mount in ZFS path
+            # and tries to restore a manual backup.
+            "rm -rf {fsname}/*",
             "mkdir -p /home/odoo/.local/share/Odoo/filestore/",
-            "cp -R /tmp/filestore {fsname}",
+            # TODO: Remove the install of `rsync` when all the customers image have installed
+            # the package, because it doesn't installed it will trigger an error.
+            cmd_filestore,
             "chown -R odoo:odoo /home/odoo/.local",
             "chmod -R o+r {fsname}"
         ]
         res = self.execute_in_auxiliary_container(database_name, bash_lines, volumes)
+        logger.info("Filestore restored")
         return res
 
     def execute_in_auxiliary_container(self, db_name, commands, volumes=False):
@@ -1398,18 +1441,48 @@ class InstanceV(DockerV):
         res = self.install_packages(apt_list, pip_list)
         return res
 
-    def clean_volumes(self):
+    def clean_volumes(self, container_name=False):
         """Removes all the files in the /tmp folder, .ssh directory, in the filestore,
         and the logs inside the container. This method is used to clean the files before
         destroying a container so they are not present the next time we deploy the same container
         """
         logger.info('Cleaning volumes')
         use_snapshot = self.__full_config.backup_type in ['btrfs', 'zfs']
-        for key, volume in self.config.get('volumes').items():
+        volumes = self._get_volumes(container_name)
+        if not volumes:
+            # If `volumes` is empty, its because the container does not exist.
+            logger.warning('Container %s does not exist, unable to clean its volumes.',
+                           container_name)
+            return
+        for key, volume in volumes.items():
             if (self.config.get('keep_db') or use_snapshot) and key == 'filestore':
                 continue
             logger.info('Removing volume: %s', volume)
-            self.exec_cmd('rm -rf {volume}'.format(volume=volume))
+            self.exec_cmd('rm -rf {volume}'.format(volume=volume),
+                          container_name=container_name)
+
+    def _get_volumes(self, container_name=False):
+        # Value used to avoid overwriting a dictionary key if two paths have the same basename
+        count = 0
+        container_name = container_name or self.config.get('container_name')
+        logger.info('Cleaning the volumes of the container: %s', container_name)
+        volumes = self.config.get('volumes').copy()
+        json_volumes = volumes.values()
+        try:
+            info = self.inspect(container_name)
+        except errors.NoSuchContainer:
+            logger.debug('Container %s does not exist, unable to obtain its volumes',
+                         container_name)
+            return {}
+        container_volumes = [volume.get('Destination') for volume in info.get('Mounts')]
+        for volume in container_volumes:
+            if volume in json_volumes:
+                continue
+            key = '_volume_%(path_name)s_%(count)s' % {
+                'path_name': path.basename(volume), 'count': count}
+            count += 1
+            volumes.update({key: volume})
+        return volumes
 
     def set_parameters(self, db_name, parameters):
         """Inserts the provided parameters in the ir.config_parameter
@@ -1545,13 +1618,15 @@ class InstanceV(DockerV):
         logger.error('Failed to restart the container, state: %s', container_obj.status)
         return False
 
-    def extras_deactivate(self):
+    def extras_deactivate(self, extra_config=False):
         """Method find the files deactivate.sql that is into of the repositories
         of the container and return a dict with the queries
         """
         res = {}
+        extra_config = extra_config or {}
         env_vars = self.config.get('env_vars') or self.docker_env
         odoo_home = env_vars.get('odoo_home')
+        env_vars.update(extra_config)
         cmd = "find {0}/instance -name 'deactivate.jinja'".format(odoo_home)
         wf_path = self.__full_config.working_folder
         name = path.basename(wf_path)
@@ -1721,7 +1796,7 @@ class InstanceV(DockerV):
                 postgres.drop(db_name)
                 return {'error': error_msg}
         extra_config = extra_config or {}
-        extra_sqls = self.extras_deactivate()
+        extra_sqls = self.extras_deactivate(extra_config)
         if extra_sqls.get('error'):
             logger.error(extra_sqls.get('error'))
             return extra_sqls
@@ -1729,9 +1804,22 @@ class InstanceV(DockerV):
                         "= 'value' and model = 'ir.config_parameter';")
         config_parameter_id = self.execute_query(db_config_user, query_config)
         config_parameter_id = config_parameter_id and config_parameter_id[0].get('id')
+        # Obtain the crons that will not be deactivated
+        action_ids = []
+        try:
+            query_res = self.execute_query(
+                db_config_user,
+                "select id from ir_act_server where model_name = 'ir.autovacuum';")
+            action_ids = [result.get('id') for result in query_res]
+        except psycopg2.errors.UndefinedColumn as error:
+            logger.warning('Unable to deactivate the crons, using legacy deactivate method: %s',
+                           error.pgerror)
+        # Parameters to render the template
         extra_config.update({
             'field_id': config_parameter_id, 'uuid': uuid4(),
-            'migration': self.__full_config.migration})
+            'migration': self.__full_config.migration,
+            'action_ids': action_ids,
+        })
         # Sqls to be executed
         template_path = deployv_static.get_template_path('deactivation.jinja')
         with open(template_path, 'r') as obj:
@@ -1743,6 +1831,8 @@ class InstanceV(DockerV):
         sql += " ,".join(["%s='%s'" % (k, v) for k, v in certificate_pac.items()])
         sqls.update({"mx_edi_certificate": sql})
         sqls.update(extra_sqls)
+        customer_id = self.__full_config.instance_config.get("customer_id")
+        customers_exclude = self.__full_config.deployer.get('ignore_timeout_deactivate', [])
         # Start to execute the queries
         try:
             for name in sorted(sqls):
@@ -1751,11 +1841,20 @@ class InstanceV(DockerV):
                 try:
                     self.execute_query(db_config_user, sql, allow_errors=True, with_as_write=True)
                 except psycopg2.errors.InsufficientPrivilege as error:
-                    logger.warning("Inssufficient permissions for user '%s': %s",
+                    logger.warning("Insufficient permissions for user '%s': %s",
                                    db_config_user.get('user'), utils.get_error_message(error))
                     self.execute_query(db_config_owner, sql, allow_errors=True)
+                except psycopg2.errors.QueryCanceled as error:
+                    if customer_id in customers_exclude:
+                        logger.warning("Timeout executing query: %s", sql)
+                    else:
+                        raise error
         except psycopg2.OperationalError as error:
-            logger.exception('Could not deactivate the database: %s', error.message)
+            if hasattr(error, 'pgerror'):
+                message = error.pgerror
+            else:
+                message = error.message
+            logger.exception('Could not deactivate the database: %s', message)
         return {}
 
     @property
@@ -1806,6 +1905,7 @@ class InstanceV(DockerV):
         :param file_name: The file's name where the repositories's information are saved.
         :type: str
         """
+        self.exec_cmd('pip3 install branchesv')
         if not self.temp_folder:
             logger.warning("Save Repositories: Doesn't exist a temp folder where save the file")
             return ''
