@@ -7,6 +7,7 @@ follows specified in the message documentation. A sender is also written with te
 
 import logging
 import pika
+import ssl
 from deployv.helpers import configuration_helper
 try:
     from configparser import ConfigParser
@@ -17,7 +18,7 @@ except ImportError:
 _logger = logging.getLogger(__name__)
 
 
-class BaseRabbitConfiguration(object):
+class BaseRabbitConfiguration:
     """ As the configuration for the sender can be from various sources this will be the base
     class for that, so there is no need to rewrite or change any code from the sender.
     Just need to be sure thar the parameters are fully filled
@@ -42,20 +43,28 @@ class BaseRabbitConfiguration(object):
                  exchange,
                  route,
                  result=False,
-                 timeout=5.0,
-                 ssl=True):
+                 timeout=None,
+                 use_ssl=True,
+                 commit=False):
         self.result_config = result
+        self.commit_config = commit
         self.user = user
         self.password = password
         self.credentials = pika.PlainCredentials(self.user, self.password)
-        self.parameters = pika.ConnectionParameters(
-            host=host,
-            heartbeat_interval=0,
-            port=int(port),
-            virtual_host=virtual_host,
-            credentials=self.credentials,
-            ssl=ssl)
-        self.parameters.socket_timeout = timeout
+        params = {
+            'host': host,
+            'heartbeat': 0,
+            'port': int(port),
+            'virtual_host': virtual_host,
+            'credentials': self.credentials
+        }
+        if use_ssl:
+            context = ssl.create_default_context()
+            context.verify_mode = ssl.CERT_REQUIRED
+            context.load_default_certs()
+            params.update({'ssl_options': pika.SSLOptions(context)})
+        self.parameters = pika.ConnectionParameters(**params)
+        self.parameters.socket_timeout = None
         self.queue_name = queue
         self.exchange_name = exchange
         self.route = route
@@ -72,7 +81,7 @@ class FileRabbitConfiguration(BaseRabbitConfiguration):
     :param result: If you need the result configuration or not, this is in the case the workers
         are going to send and ack, result or status message
     """
-    def __init__(self, config_object, result=False, status=False):
+    def __init__(self, config_object, worker_type=False):
         if isinstance(config_object, ConfigParser):
             self.config = configuration_helper.DeployvConfig(worker_config=config_object)
         elif isinstance(config_object, configuration_helper.DeployvConfig):
@@ -80,6 +89,7 @@ class FileRabbitConfiguration(BaseRabbitConfiguration):
         else:
             raise ValueError('No valid config object provided')
         self.config = config_object
+        self.wtype = worker_type
         self._config_object = config_object
         user = self.config.rmq.get('rmq_user')
         passwd = self.config.rmq.get('rmq_passwd')
@@ -88,16 +98,18 @@ class FileRabbitConfiguration(BaseRabbitConfiguration):
         vhost = self.config.rmq.get('rmq_vhost')
         timeout = self.config.rmq.get('rmq_timeout')
         exchange = self.config.rmq.get('rmq_exchange')
-        ssl = self.config.rmq.get('rmq_ssl')
-        values = (
-            result and ('rmq_stat_queue', 'rmq_stat_topic') or
-            status and ('rmq_stat_queue', 'rmq_status_stat_topic') or
-            ('rmq_task_queue', 'rmq_task_topic'))
+        use_ssl = self.config.rmq.get('rmq_ssl')
+        keys = {'status': ('rmq_stat_queue', 'rmq_status_stat_topic'),
+                'result': ('rmq_stat_queue', 'rmq_stat_topic'),
+                'commit': ('rmq_task_queue', 'rmq_commit_topic'),
+                }
+        values = keys.get(worker_type, ('rmq_task_queue', 'rmq_task_topic'))
         queue = self.config.rmq.get(values[0])
         route = self.config.rmq.get(values[1])
-
-        super(FileRabbitConfiguration, self).__init__(host, port, user, passwd, vhost, queue,
-                                                      exchange, route, result, timeout, ssl)
+        result = worker_type == 'result'
+        super(FileRabbitConfiguration, self).__init__(
+            host, port, user, passwd, vhost, queue, exchange, route, result, timeout,
+            use_ssl, worker_type == 'commit')
 
     def get_result_object(self):
-        return FileRabbitConfiguration(self._config_object, result=True)
+        return FileRabbitConfiguration(self._config_object, 'result')
