@@ -1,9 +1,6 @@
 # coding: utf-8
-from deployv.helpers.utils import get_error_message
 import logging
 import pika
-from random import randint
-from time import sleep
 
 
 _logger = logging.getLogger(__name__)
@@ -14,7 +11,7 @@ class ReconnectionException(Exception):
         super(ReconnectionException, self).__init__(message, *args, **kwargs)
 
 
-class RabbitReceiverV(object):
+class RabbitReceiverV:
     """ This is an example consumer that will handle unexpected interactions
     with RabbitMQ such as channel and connection closures.
 
@@ -36,12 +33,17 @@ class RabbitReceiverV(object):
         :param str amqp_url: The AMQP url to connect with
 
         """
+        self.should_reconnect = False
         self._connection = None
         self._channel = None
         self._closing = False
         self._consumer_tag = None
+        self._consuming = False
+
         self._config = listener_configuration
-        if self._config.result_config:
+        if self._config.commit_config:
+            self._binding_key = '%s.commit' % (self._config.route)
+        elif self._config.result_config:
             self._binding_key = self._config.route
         else:
             self._binding_key = '{base}.{wid}'.format(base=self._config.route, wid=worker_id)
@@ -54,12 +56,23 @@ class RabbitReceiverV(object):
         :rtype: pika.SelectConnection
 
         """
-        _logger.info('Connecting to %s:%s',
-                     self._config.parameters.host,
-                     self._config.parameters.port)
-        return pika.SelectConnection(self._config.parameters,
-                                     self.on_connection_open,
-                                     stop_ioloop_on_close=False)
+        _logger.debug('Connecting to %s:%s',
+                      self._config.parameters.host,
+                      self._config.parameters.port)
+        return pika.SelectConnection(
+                parameters=self._config.parameters,
+                on_open_callback=self.on_connection_open,
+                on_open_error_callback=self.on_connection_open_error,
+                on_close_callback=self.on_connection_closed)
+
+    def close_connection(self):
+        """ This method closes the connection to RabbitMQ."""
+        self._consuming = False
+        if self._connection.is_closing or self._connection.is_closed:
+            _logger.debug('Connection already closed')
+            return
+        _logger.debug('Closing connection')
+        self._connection.close()
 
     def on_connection_open(self, unused_connection):
         """ This method is called by pika once the connection to RabbitMQ has
@@ -69,19 +82,19 @@ class RabbitReceiverV(object):
         :type unused_connection: pika.SelectConnection
 
         """
-        _logger.info('Connection opened')
-        self.add_on_connection_close_callback()
+        _logger.debug('Connection opened')
         self.open_channel()
 
-    def add_on_connection_close_callback(self):
-        """ This method adds an on close callback that will be invoked by pika
-        when RabbitMQ closes the connection to the publisher unexpectedly.
-
+    def on_connection_open_error(self, _unused_connection, err):
+        """This method is called by pika if the connection to RabbitMQ
+        can't be established.
+        :param pika.SelectConnection _unused_connection: The connection
+        :param Exception err: The error
         """
-        _logger.info('Adding connection close callback')
-        self._connection.add_on_close_callback(self.on_connection_closed)
+        _logger.error('Connection open failed: %s', err)
+        self.reconnect()
 
-    def on_connection_closed(self, connection, reply_code, reply_text):
+    def on_connection_closed(self, connection, reason):
         """ This method is invoked by pika when the connection to RabbitMQ is
         closed unexpectedly. Since it is unexpected, we will reconnect to
         RabbitMQ if it disconnects.
@@ -95,9 +108,8 @@ class RabbitReceiverV(object):
         if self._closing:
             self._connection.ioloop.stop()
         else:
-            _logger.warning('Connection closed, reopening in %s seconds: (%s) %s',
-                            self._config.parameters.socket_timeout, reply_code, reply_text)
-            self._connection.add_timeout(self._config.parameters.socket_timeout, self.reconnect)
+            _logger.debug('Connection closed: %s, reopening', reason)
+            self.reconnect()
 
     def reconnect(self):
         """ Will be invoked by the IOLoop timer if the connection is
@@ -105,7 +117,8 @@ class RabbitReceiverV(object):
 
         """
         # This is the old connection IOLoop instance, stop its ioloop
-        raise ReconnectionException('Disconnected from rabbit server')
+        self.should_reconnect = True
+        self.stop()
 
     def open_channel(self):
         """ Open a new channel with RabbitMQ by issuing the Channel.Open RPC
@@ -113,7 +126,7 @@ class RabbitReceiverV(object):
         on_channel_open callback will be invoked by pika.
 
         """
-        _logger.info('Creating a new channel')
+        _logger.debug('Creating a new channel')
         self._connection.channel(on_open_callback=self.on_channel_open)
 
     def on_channel_open(self, channel):
@@ -125,9 +138,8 @@ class RabbitReceiverV(object):
         :param pika.channel.Channel channel: The channel object
 
         """
-        _logger.info('Channel opened')
+        _logger.debug('Channel opened')
         self._channel = channel
-        self._channel.basic_qos(prefetch_count=1)
         self.add_on_channel_close_callback()
         self.setup_exchange(self._config.exchange_name)
 
@@ -136,10 +148,10 @@ class RabbitReceiverV(object):
         RabbitMQ unexpectedly closes the channel.
 
         """
-        _logger.info('Adding channel close callback')
+        _logger.debug('Adding channel close callback')
         self._channel.add_on_close_callback(self.on_channel_closed)
 
-    def on_channel_closed(self, channel, reply_code, reply_text):
+    def on_channel_closed(self, channel, reason):
         """ Invoked by pika when RabbitMQ unexpectedly closes the channel.
         Channels are usually closed if you attempt to do something that
         violates the protocol, such as re-declare an exchange or queue with
@@ -151,9 +163,8 @@ class RabbitReceiverV(object):
         :param str reply_text: The text reason the channel was closed
 
         """
-        _logger.warning('Channel %i was closed: (%s) %s',
-                        channel, reply_code, reply_text)
-        self._connection.close()
+        _logger.debug('Channel %i was closed: %s', channel, reason)
+        self.close_connection()
 
     def setup_exchange(self, exchange_name):
         """ Setup the exchange on RabbitMQ by invoking the Exchange.Declare RPC
@@ -163,10 +174,10 @@ class RabbitReceiverV(object):
         :param str|unicode exchange_name: The name of the exchange to declare
 
         """
-        _logger.info('Declaring exchange %s', exchange_name)
-        self._channel.exchange_declare(self.on_exchange_declareok,
-                                       exchange_name,
-                                       self.EXCHANGE_TYPE)
+        _logger.debug('Declaring exchange %s', exchange_name)
+        self._channel.exchange_declare(exchange_name,
+                                       self.EXCHANGE_TYPE,
+                                       callback=self.on_exchange_declareok)
 
     def on_exchange_declareok(self, unused_frame):
         """ Invoked by pika when RabbitMQ has finished the Exchange.Declare RPC
@@ -175,7 +186,7 @@ class RabbitReceiverV(object):
         :param pika.Frame.Method unused_frame: Exchange.DeclareOk response frame
 
         """
-        _logger.info('Exchange declared')
+        _logger.debug('Exchange declared')
         self.setup_queue(self._binding_key)
 
     def setup_queue(self, queue_name):
@@ -186,8 +197,8 @@ class RabbitReceiverV(object):
         :param str|unicode queue_name: The name of the queue to declare.
 
         """
-        _logger.info('Declaring queue %s', queue_name)
-        self._channel.queue_declare(self.on_queue_declareok, queue=queue_name, durable=True)
+        _logger.debug('Declaring queue %s', queue_name)
+        self._channel.queue_declare(queue_name, callback=self.on_queue_declareok, durable=True)
 
     def on_queue_declareok(self, method_frame):
         """ Method invoked by pika when the Queue.Declare RPC call made in
@@ -199,16 +210,14 @@ class RabbitReceiverV(object):
         :param pika.frame.Method method_frame: The Queue.DeclareOk frame
 
         """
-        _logger.info('Binding %s to %s with %s',
-                     self._config.exchange_name, self._config.queue_name, self._binding_key)
-        self._channel.queue_bind(self.on_bindok, queue=self._binding_key,
-                                 exchange=self._config.exchange_name,
-                                 routing_key=self._binding_key)
-        _logger.info('Binding %s to %s with %s',
-                     self._config.exchange_name, self._config.queue_name, self._config.route)
-        self._channel.queue_bind(self.on_bindok, queue=self._config.queue_name,
-                                 exchange=self._config.exchange_name,
-                                 routing_key=self._config.route)
+        _logger.debug('Binding %s to %s with %s',
+                      self._config.exchange_name, self._config.queue_name, self._binding_key)
+        self._channel.queue_bind(self._binding_key, self._config.exchange_name,
+                                 self._binding_key, callback=self.on_bindok)
+        _logger.debug('Binding %s to %s with %s',
+                      self._config.exchange_name, self._config.queue_name, self._config.route)
+        self._channel.queue_bind(self._config.queue_name, self._config.exchange_name,
+                                 self._config.route, callback=self.on_bindok)
 
     def on_bindok(self, unused_frame):
         """ Invoked by pika when the Queue.Bind method has completed. At this
@@ -218,7 +227,25 @@ class RabbitReceiverV(object):
         :param pika.frame.Method unused_frame: The Queue.BindOk response frame
 
         """
-        _logger.info('Queue bound')
+        _logger.debug('Queue bound')
+        self.set_qos()
+
+    def set_qos(self):
+        """This method sets up the consumer prefetch to only be delivered
+        one message at a time. The consumer must acknowledge this message
+        before RabbitMQ will deliver another one. You should experiment
+        with different prefetch values to achieve desired performance.
+        """
+        self._channel.basic_qos(prefetch_count=1, callback=self.on_basic_qos_ok)
+
+    def on_basic_qos_ok(self, _unused_frame):
+        """Invoked by pika when the Basic.QoS method has completed. At this
+        point we will start consuming messages by calling start_consuming
+        which will invoke the needed RPC commands to start the process.
+        :param pika.frame.Method _unused_frame: The Basic.QosOk response frame
+        """
+        # TODO: if we need this, remember to set a configurable value
+        _logger.debug('QOS set to: 1')
         self.start_consuming()
 
     def start_consuming(self):
@@ -227,14 +254,14 @@ class RabbitReceiverV(object):
         cancels the consumer. It then issues the Basic.Consume RPC command
         which returns the consumer tag that is used to uniquely identify the
         consumer with RabbitMQ. We keep the value to use it when we want to
-        cancel consuming. The on_message method is passed in as a callback pika
-        will invoke when a message is fully received.
-
+        cancel consuming.
         """
-        _logger.info('Issuing consumer related RPC commands')
+        _logger.debug('Issuing consumer related RPC commands')
         self.add_on_cancel_callback()
-        self._consumer_tag = self._channel.basic_consume(self._callback_funct,
-                                                         self._binding_key)
+        self._consumer_tag = self._channel.basic_consume(
+            self._binding_key, self._callback_funct,
+            callback=self.stop_consuming)
+        self._consuming = True
 
     def add_on_cancel_callback(self):
         """ Add a callback that will be invoked if RabbitMQ cancels the consumer
@@ -242,7 +269,7 @@ class RabbitReceiverV(object):
         on_consumer_cancelled will be invoked by pika.
 
         """
-        _logger.info('Adding consumer cancellation callback')
+        _logger.debug('Adding consumer cancellation callback')
         self._channel.add_on_cancel_callback(self.on_consumer_cancelled)
 
     def on_consumer_cancelled(self, method_frame):
@@ -257,42 +284,14 @@ class RabbitReceiverV(object):
         if self._channel:
             self._channel.close()
 
-    def on_message(self, unused_channel, basic_deliver, properties, body):
-        """ Invoked by pika when a message is delivered from RabbitMQ. The
-        channel is passed for your convenience. The basic_deliver object that
-        is passed in carries the exchange, routing key, delivery tag and
-        a redelivered flag for the message. The properties passed in is an
-        instance of BasicProperties with the message properties and the body
-        is the message that was sent.
-
-        :param pika.channel.Channel unused_channel: The channel object
-        :param pika.Spec.Basic.Deliver: basic_deliver method
-        :param pika.Spec.BasicProperties: properties
-        :param str|unicode body: The message body
-
-        """
-        _logger.info('Received message # %s from %s: %s',
-                     basic_deliver.delivery_tag, properties.app_id, body)
-        self.acknowledge_message(basic_deliver.delivery_tag)
-
-    def acknowledge_message(self, delivery_tag):
-        """ Acknowledge the message delivery from RabbitMQ by sending a
-        Basic.Ack RPC method for the delivery tag.
-
-        :param int delivery_tag: The delivery tag from the Basic.Deliver frame
-
-        """
-        _logger.info('Acknowledging message %s', delivery_tag)
-        self._channel.basic_ack(delivery_tag)
-
-    def stop_consuming(self):
+    def stop_consuming(self, *args, **kwargs):
         """ Tell RabbitMQ that you would like to stop consuming by sending the
         Basic.Cancel RPC command.
 
         """
         if self._channel:
-            _logger.info('Sending a Basic.Cancel RPC command to RabbitMQ')
-            self._channel.basic_cancel(self.on_cancelok, self._consumer_tag)
+            _logger.debug('Sending a Basic.Cancel RPC command to RabbitMQ')
+            self._channel.basic_cancel(self._consumer_tag, self.on_cancelok)
 
     def on_cancelok(self, unused_frame):
         """ This method is invoked by pika when RabbitMQ acknowledges the
@@ -303,7 +302,8 @@ class RabbitReceiverV(object):
         :param pika.frame.Method unused_frame: The Basic.CancelOk frame
 
         """
-        _logger.info('RabbitMQ acknowledged the cancellation of the consumer')
+        self._consuming = False
+        _logger.debug('RabbitMQ acknowledged the cancellation of the consumer')
         self.close_channel()
 
     def close_channel(self):
@@ -311,7 +311,7 @@ class RabbitReceiverV(object):
         Channel.Close RPC command.
 
         """
-        _logger.info('Closing the channel')
+        _logger.debug('Closing the channel')
         self._channel.close()
 
     def run(self, callback_funct):
@@ -319,22 +319,9 @@ class RabbitReceiverV(object):
         starting the IOLoop to block and allow the SelectConnection to operate.
 
         """
-        if self._connection and self._connection.is_open:
-            return
         self._callback_funct = callback_funct
-        while not self._closing:
-            try:
-                self._connection = self.connect()
-                self._connection.ioloop.start()
-            except ReconnectionException:
-                self._connection.ioloop.stop()
-            except Exception as error:  # pylint: disable=W0703
-                _logger.error('Error connecting to Rabbit server')
-                error_msg = get_error_message(error)
-                _logger.exception(error_msg)
-                wait = randint(3, 7)
-                _logger.error('Waiting %s seconds before retry', wait)
-                sleep(wait)
+        self._connection = self.connect()
+        self._connection.ioloop.start()
 
     def stop(self):
         """ Cleanly shutdown the connection to RabbitMQ by stopping the consumer
@@ -347,15 +334,12 @@ class RabbitReceiverV(object):
         the IOLoop will be buffered but not processed.
 
         """
-        if not self._connection or self._connection.is_closed:
-            return
-        _logger.info('Stopping')
-        self._closing = True
-        self.stop_consuming()
-        self._connection.ioloop.start()
-        _logger.info('Stopped')
-
-    def close_connection(self):
-        """ This method closes the connection to RabbitMQ."""
-        _logger.info('Closing connection')
-        self._connection.close()
+        _logger.debug('Stopping')
+        if not self._closing:
+            self._closing = True
+            if self._consuming:
+                self.stop_consuming()
+                self._connection.ioloop.start()
+            else:
+                self._connection.ioloop.stop()
+        _logger.debug('Stopped')
